/**
 * Define api of apeman-api-kv
 * @function define
 * @param {string} filename - Filename of store
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const aslogger = require('aslogger')
const akv = require('akv')
const spec = require('./spec')

/** @lends define */
function define (filename, options = {}) {
  let logger = aslogger({ disabled: !options.logging })
  let store = akv(filename)

  return {
    /**
     * Test the reachability of the api.
     * @method ping
     * @param {string} [pong='pong'] - Pong message to return
     * @returns {string} - Pong message
     */
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    /**
     * Touch kv file
     * @method touch
     * @returns {Promise}
     */
    touch () {
      return co(function * () {
        yield store.touch()
      })
    },

    /**
     * Get value with key
     * @method get
     * @param {string} key
     * @returns {Promise}
     */
    get (key) {
      return co(function * () {
        return yield store.get(key)
      })
    },

    /**
     * List keys
     * @method keys
     * @returns {Promise.<string[]>}
     */
    keys () {
      return co(function * () {
        return yield store.keys()
      })
    },

    /**
     * Get all values
     * @method all
     * @returns {Promise.<Object>}
     */
    all () {
      return co(function * () {
        return yield store.all()
      })
    },

    /**
     * Set value
     * @method set
     * @param {string} key - Key to set
     * @param {*} value - Value to set
     * @returns {Promise}
     */
    set (key, value) {
      return co(function * () {
        yield store.set(key, value)
      })
    },

    /**
     * @method destroy
     * @returns {Promise}
     */
    destroy () {
      return co(function * () {
        yield store.destroy()
      })
    },

    /**
     * @method commit
     * @returns {Promise}
     */
    commit () {
      return co(function * () {
        yield store.commit()
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: spec
  }
}

module.exports = define
